@@ -37,7 +37,7 @@ BEGIN {
 }
 
 # ABSTRACT: Perl bindings to libarchive via FFI
-our $VERSION = '0.05'; # VERSION
+our $VERSION = '0.06'; # VERSION
 
 ffi_lib do {
   my $file = locate_module_share_lib();
@@ -47,7 +47,6 @@ ffi_lib do {
 ffi_lib(Alien::Libarchive->new);
 
 require Archive::Libarchive::FFI::Constant;
-require Archive::Libarchive::FFI::Callback;
 
 $Archive::Libarchive::FFI::on_attach ||= sub {};
 
@@ -63,7 +62,17 @@ sub _attach ($$$)
 {
   $Archive::Libarchive::FFI::on_attach->(@_);
   my($name, $arg, $ret) = @_;
-  $name = [ $name => "_$name" ] if grep { $_ == FFI::Raw::str } ($ret, @$arg);
+  if(grep { $_ == FFI::Raw::str } ($ret, @$arg))
+  {
+    if(ref $name)
+    {
+      $name->[1] = "_" . $name->[1];
+    }
+    else
+    {
+      $name = [ $name => "_$name" ] 
+    }
+  }
   if($ret == _void)
   {
     _attach_function $name, $arg, $ret, sub {
@@ -79,6 +88,9 @@ sub _attach ($$$)
 }
 
 _attach 'archive_version_number',                        undef, _int;
+
+require Archive::Libarchive::FFI::Callback;
+
 _attach 'archive_version_string',                        undef, _str;
 _attach 'archive_clear_error',                           [ _ptr ], _void;
 _attach 'archive_copy_error',                            [ _ptr ], _int;
@@ -87,10 +99,22 @@ _attach 'archive_file_count',                            [ _ptr ], _int;
 _attach 'archive_format',                                [ _ptr ], _int;
 _attach 'archive_format_name',                           [ _ptr ], _str;
 _attach 'archive_seek_data',                             [ _ptr, _int64, _int ], _int64;
-_attach 'archive_error_string',                          [ _ptr ], _str;
+
+if(archive_version_number() >= 3000000)
+{
+  _attach 'archive_error_string',                          [ _ptr ], _str;
+}
+else
+{
+  _attach_function [ 'archive_error_string' => '_archive_error_string' ], [ _ptr ], _str, sub {
+    my($sub, $archive) = @_;
+    my $ret = $sub->($archive);
+    return if $ret eq '(Empty error message)';
+    return $ret;
+  };
+}
 
 _attach 'archive_read_new',                              undef, _ptr;
-_attach 'archive_read_support_filter_all',               [ _ptr ], _int;
 _attach 'archive_read_support_format_all',               [ _ptr ], _int;
 _attach 'archive_read_open1',                            [ _ptr ], _int;
 _attach 'archive_read_open_filename',                    [ _ptr, _str, _int ], _int;
@@ -98,7 +122,6 @@ _attach 'archive_read_data_skip',                        [ _ptr ], _int;
 _attach 'archive_read_close',                            [ _ptr ], _int;
 _attach 'archive_read_append_filter',                    [ _ptr, _int ], _int;
 _attach 'archive_read_append_filter_program',            [ _ptr, _str ], _int;
-_attach 'archive_read_support_filter_program',           [ _ptr, _str ], _int;
 _attach 'archive_read_support_format_by_code',           [ _ptr, _int ], _int;
 _attach 'archive_read_header_position',                  [ _ptr ], _int64;
 _attach 'archive_read_set_filter_option',                [ _ptr, _str, _str, _str ], _int;
@@ -111,6 +134,16 @@ _attach 'archive_read_extract',                          [ _ptr, _ptr, _int ], _
 _attach 'archive_read_extract2',                         [ _ptr, _ptr, _ptr ], _int;
 _attach 'archive_read_extract_set_skip_file',            [ _ptr, _int64, _int64 ], _void;
 
+if(archive_version_number() >= 3000000)
+{
+  _attach 'archive_read_support_filter_program', [ _ptr, _str ], _int;
+}
+else
+{
+  _attach ['archive_read_support_compression_program' => 'archive_read_support_filter_program'], [ _ptr, _str ], _int;
+}
+
+
 _attach 'archive_filter_code',                           [ _ptr, _int ], _int;
 _attach 'archive_filter_count',                          [ _ptr ], _int;
 _attach 'archive_filter_name',                           [ _ptr, _int ], _str;
@@ -119,7 +152,6 @@ _attach 'archive_filter_bytes',                          [ _ptr, _int ], _int64;
 _attach 'archive_write_new',                             undef, _ptr;
 _attach 'archive_write_add_filter',                      [ _ptr, _int ], _int;
 _attach 'archive_write_add_filter_by_name',              [ _ptr, _str ], _int;
-_attach 'archive_write_add_filter_program',              [ _ptr, _str ], _int;
 _attach 'archive_write_set_format',                      [ _ptr, _int ], _int;
 _attach 'archive_write_set_format_by_name',              [ _ptr, _str ], _int;
 _attach 'archive_write_open_filename',                   [ _ptr, _str ], _int;
@@ -145,6 +177,16 @@ _attach 'archive_write_get_bytes_per_block',             [ _ptr ], _int;
 _attach 'archive_write_set_bytes_in_last_block',         [ _ptr, _int ], _int;
 _attach 'archive_write_set_bytes_per_block',             [ _ptr, _int ], _int;
 
+if(archive_version_number() >= 3000000)
+{
+  _attach 'archive_write_add_filter_program', [ _ptr, _str ], _int;
+}
+else
+{
+  _attach ['archive_write_set_compression_program' => 'archive_write_add_filter_program'], [ _ptr, _str ], _int;
+}
+
+
 _attach 'archive_entry_clear',                           [ _ptr ], _void;
 _attach 'archive_entry_clone',                           [ _ptr ], _ptr;
 _attach 'archive_entry_free',                            [ _ptr ], _void;
@@ -211,7 +253,6 @@ _attach 'archive_entry_uid',                             [ _ptr ], _uid_t;
 _attach 'archive_entry_copy_sourcepath',                 [ _ptr, _str ], _void;
 _attach 'archive_entry_acl',                             [ _ptr ], _ptr;
 _attach 'archive_entry_acl_clear',                       [ _ptr ], _int;
-_attach 'archive_entry_acl_add_entry',                   [ _ptr, _int, _int, _int, _int, _str ], _int;
 _attach 'archive_entry_acl_reset',                       [ _ptr, _int ], _int;
 _attach 'archive_entry_acl_text',                        [ _ptr, _int ], _str;
 _attach 'archive_entry_acl_count',                       [ _ptr, _int ], _int;
@@ -220,6 +261,18 @@ _attach 'archive_entry_sparse_add_entry',                [ _ptr, _int64, _int64
 _attach 'archive_entry_sparse_count',                    [ _ptr ], _int;
 _attach 'archive_entry_sparse_reset',                    [ _ptr ], _int;
 
+if(archive_version_number() >= 3000000)
+{
+  _attach 'archive_entry_acl_add_entry',                   [ _ptr, _int, _int, _int, _int, _str ], _int;
+}
+else
+{
+  _attach_function [ 'archive_entry_acl_add_entry' => '_archive_entry_acl_add_entry' ], [ _ptr, _int, _int, _int, _int, _str ], _void, sub {
+    shift->(@_);
+    ARCHIVE_OK();
+  };
+}
+
 _attach 'archive_entry_linkresolver_free',               [ _ptr ], _void;
 _attach 'archive_entry_linkresolver_new',                undef, _ptr;
 _attach 'archive_entry_linkresolver_set_strategy',       [ _ptr, _int ], _void;
@@ -259,12 +312,35 @@ _attach 'archive_match_include_file_time',               [ _ptr, _int, _str ], _
 _attach 'archive_match_include_time',                    [ _ptr, _int, _time_t, _long ], _int;
 _attach 'archive_match_path_unmatched_inclusions',       [ _ptr ], _int;
 
-_attach "archive_read_support_filter_$_",  [ _ptr ], _int
-  for qw( bzip2 compress gzip grzip lrzip lzip lzma lzop none rpm uu xz );
+foreach my $type (qw( all bzip2 compress gzip grzip lrzip lzip lzma lzop none rpm uu xz ))
+{
+  my $name = "archive_read_support_filter_$type";
+  eval {
+    attach_function $name, [ _ptr ], _int;
+  };
+  if($@)
+  {
+    my $real = "archive_read_support_compression_$type";
+    eval { attach_function [ $real => $name ], [ _ptr ], _int };
+  }
+}
+
 _attach "archive_read_support_format_$_",  [ _ptr ], _int
   for qw( 7zip ar cab cpio empty gnutar iso9660 lha mtree rar raw tar xar zip );
-_attach "archive_write_add_filter_$_", [ _ptr ], _int
-  for qw( b64encode bzip2 compress grzip gzip lrzip lzip lzma lzop none uuencode xz );
+
+foreach my $type (qw( b64encode bzip2 compress grzip gzip lrzip lzip lzma lzop none uuencode xz ))
+{
+  my $name = "archive_write_add_filter_$type";
+  eval {
+    attach_function $name, [ _ptr ], _int;
+  };
+  if($@)
+  {
+    my $real = "archive_write_set_compression_$type";
+    eval { attach_function [ $real => $name ], [ _ptr ], _int };
+  }
+}  
+
 _attach "archive_write_set_format_$_", [ _ptr ], _int
   for qw( 7zip ar_bsd ar_svr4 cpio cpio_newc gnutar iso9660 mtree mtree_classic 
           pax pax_restricted shar shar_dump ustar v7tar xar zip);
@@ -447,7 +523,9 @@ _attach_function $_, [ _ptr, _str, _ptr, _size_t ],_int, sub
 {
   my($cb, $archive, $command, $signature) = @_;
   $cb->($archive, $command, scalar_to_buffer($signature));
-} for qw( archive_read_append_filter_program_signature archive_read_support_filter_program_signature );
+} for (
+  'archive_read_append_filter_program_signature',
+  archive_version_number() >= 3000000 ? 'archive_read_support_filter_program_signature' : [ archive_read_support_compression_program_signature => 'archive_read_support_filter_program_signature']);
 
 # this is an unusual one which doesn't need to be decoded
 # because it should always be ASCII
@@ -465,7 +543,8 @@ _attach_function 'archive_entry_linkify', [ _ptr, _ptr, _ptr ], _void, sub
 
 _attach_function [ 'archive_entry_copy_fflags_text' => '_archive_entry_set_fflags_text' ], [ _ptr, _str ], _void, sub
 {
-  shift->(@_);
+  my($sub, $entry, $text) = @_;
+  $sub->($entry, $text);
   ARCHIVE_OK();
 };
 
@@ -543,7 +622,7 @@ Archive::Libarchive::FFI - Perl bindings to libarchive via FFI
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
